import React, { useCallback, useEffect, useMemo, useState } from "react";
import { useParams, useLocation, useNavigate } from "react-router-dom";
import { SnackbarProvider, enqueueSnackbar } from "notistack";

import { Box, Card, CardContent, Grid, Typography } from "@mui/material";
import LoadingButton from "@mui/lab/LoadingButton/LoadingButton";
import {
  MsgOnboardParticipantEncodeObject,
  typeUrlMsgOnboardParticipant,
} from "@cerc-io/registry-sdk";
import { StargateClient } from "@cosmjs/stargate";

import { useWalletConnectContext } from "../context/WalletConnectContext";

const SignWithCosmos = () => {
  const { session, signClient } = useWalletConnectContext();

  const { cosmosAddress, ethSignature } = useParams();

  const [isLoading, setIsLoading] = useState(false);
  const [balance, setBalance] = useState('');
  const [isRequesting, setIsRequesting] = useState(false);

  const navigate = useNavigate();
  const location = useLocation();
  const innerMessage = location.state;
  const ethAddress = innerMessage.address;

  const createCosmosClient = useCallback(async (endpoint: string) => {
    return await StargateClient.connect(endpoint);
  }, []);

  const onboardParticipantMsg: MsgOnboardParticipantEncodeObject =
    useMemo(() => {
      return {
        typeUrl: typeUrlMsgOnboardParticipant,
        value: {
          participant: cosmosAddress,
          ethPayload: innerMessage,
          ethSignature,
        },
      };
    }, [cosmosAddress, innerMessage, ethSignature]);

  const handleTokenRequest = async () => {
    try {
      setIsRequesting(true);
      const response = await fetch(`${process.env.REACT_APP_FAUCET_ENDPOINT!}/faucet`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          address: cosmosAddress,
        }),
      });

      if (response.ok) {
        enqueueSnackbar('Tokens sent successfully',  { variant: "success" });
      } else {
        const errorResponse = await response.json();
        if (response.status === 429) {
          enqueueSnackbar(`${response.statusText} : ${errorResponse.error}`, { variant: "error" });
        } else {
          throw new Error(errorResponse.error);
        }
      }

      getBalances();
    } catch (error) {
      console.error(error);
      enqueueSnackbar("Error getting tokens from faucet", { variant: "error" });
    } finally {
      setIsRequesting(false);
    }
  };

  const sendTransaction = async (
    transactionMessage: MsgOnboardParticipantEncodeObject
  ) => {
    if (!ethAddress) {
      enqueueSnackbar("Set nitro address");
      return;
    }

    try {
      setIsLoading(true);

      const params = { transactionMessage, signer: cosmosAddress };
      const responseFromWallet = await signClient!.request<{
        code: number;
      }>({
        topic: session!.topic,
        chainId: `cosmos:${process.env.REACT_APP_LACONICD_CHAIN_ID}`,
        request: {
          method: "cosmos_sendTransaction",
          params,
        },
      });
      if (responseFromWallet.code !== 0) {
        enqueueSnackbar("Transaction not sent", { variant: "error" });
      } else {
        navigate(`/onboarding-success/${cosmosAddress}`);
      }
    } catch (error) {
      console.error(error);
      enqueueSnackbar("Error in sending transaction", { variant: "error" });
    } finally {
      setIsLoading(false);
    }
  };

  const getBalances = useCallback(async () => {
    try {
      const cosmosClient = await createCosmosClient(process.env.REACT_APP_LACONICD_RPC_ENDPOINT!);
      const balance = await cosmosClient.getBalance(cosmosAddress!, process.env.REACT_APP_LACONICD_DENOM!);
      setBalance(balance.amount);
    } catch (error) {
      console.error('Error fetching balance:', error);
      throw error;
    }
  }, [cosmosAddress, createCosmosClient]);

  useEffect(() => {
    getBalances();
  }, [getBalances]);

  return (
    <Box
      sx={{
        display: "flex",
        flexDirection: "column",
        marginTop: "100px",
        gap: "10px",
      }}
    >
      <Typography variant="h5">Send transaction to chain</Typography>
      <Typography>Cosmos Account:</Typography>
      <Card className='mt-1 mb-1'>
        <CardContent>
          <Grid container spacing={2}>
            <Grid item xs={9}>
              <Typography variant="body1">Address: {cosmosAddress}</Typography>
              <Typography variant="body1">Balance: {balance} {process.env.REACT_APP_LACONICD_DENOM}</Typography>
            </Grid>
            <Grid item xs={3} container justifyContent="flex-end">
              <LoadingButton
                variant="contained"
                onClick={handleTokenRequest}
                disabled={isRequesting}
                loading={isRequesting}
              >
                Request tokens from Faucet
              </LoadingButton>
            </Grid>
          </Grid>
        </CardContent>
      </Card>
      <Typography variant="body1">
        Onboarding message: <br />
      </Typography>
      <Box
        sx={{
          backgroundColor: "lightgray",
          padding: 3,
          wordWrap: "break-word",
        }}
      >
        <pre style={{ whiteSpace: "pre-wrap", margin: 0 }}>
          {JSON.stringify(onboardParticipantMsg, null, 2)}{" "}
        </pre>
      </Box>

      <Box>
        <LoadingButton
          variant="contained"
          onClick={async () => {
            await sendTransaction(onboardParticipantMsg);
          }}
          loading={isLoading}
          disabled={balance === '0'}
        >
          Send transaction
        </LoadingButton>
      </Box>
      <SnackbarProvider />
    </Box>
  );
};

export default SignWithCosmos;
